from spaceone.core.error import *


class ERROR_NOT_SUPPORTED_MFA_TYPE(ERROR_INVALID_ARGUMENT):
    _message = "Not supported MFA type. (support_mfa_types = {support_mfa_types})"


class ERROR_MFA_ALREADY_ENABLED(ERROR_UNKNOWN):
    _message = "MFA already enabled. (user_id = {user_id})"


class ERROR_MFA_ALREADY_DISABLED(ERROR_UNKNOWN):
    _message = "MFA already disabled. (user_id = {user_id})"


class ERROR_MFA_REQUIRED(ERROR_UNKNOWN):
    _message = "MFA is required. (user_id = {user_id})"


class ERROR_MFA_NOT_ENABLED(ERROR_UNKNOWN):
    _message = "MFA is not enabled. (user_id = {user_id})"



from spaceone.core.error import *


class ERROR_DOMAIN_STATE(ERROR_INVALID_ARGUMENT):
    _message = "Domain is disabled. (domain_id = {domain_id})"


class ERROR_DOMAIN_ADMIN_ROLE_IS_NOT_DEFINED(ERROR_UNKNOWN):
    _message = "Domain admin role is not defined."



from mongoengine import *
from spaceone.core.model.mongo_model import MongoModel


class APIKey(MongoModel):
    api_key_id = StringField(max_length=40, generate_id="api-key", unique=True)
    name = StringField(max_length=40, default=None, null=True)
    state = StringField(
        max_length=20, default="ENABLED", choices=("ENABLED", "DISABLED", "EXPIRED")
    )
    owner_type = StringField(max_length=20, choices=("USER", "APP"))
    user = ReferenceField("User", null=True, default=None, reverse_delete_rule=CASCADE)
    app = ReferenceField("App", null=True, default=None, reverse_delete_rule=CASCADE)
    user_id = StringField(max_length=40, default=None, null=True)
    app_id = StringField(max_length=40, default=None, null=True)
    domain_id = StringField(max_length=40, required=True)
    created_at = DateTimeField(auto_now_add=True)
    last_accessed_at = DateTimeField(auto_now_add=True)
    expired_at = DateTimeField(default=None, null=True)

    meta = {
        "updatable_fields": [
            "state",
            "name",
            "last_accessed_at"
        ],
        "minimal_fields": [
            "api_key_id",
            "state",
            "user_id"
        ],
        "ordering": ["api_key_id"],
        "indexes": [
            "state",
            "owner_type",
            "user_id",
            "app_id",
            "domain_id",
            "last_accessed_at"
        ],
    }



from mongoengine import *
from spaceone.core.model.mongo_model import MongoModel


class PagePermission(EmbeddedDocument):
    page = StringField(required=True)
    permission = StringField(max_length=20, choices=('VIEW', 'MANAGE'), required=True)


class Role(MongoModel):
    role_id = StringField(max_length=40, required=True, unique_with='domain_id')
    name = StringField(max_length=255, unique_with='domain_id')
    version = StringField(max_length=40, default=None, null=True)
    role_type = StringField(
        max_length=20,
        choices=('SYSTEM', 'SYSTEM_ADMIN', 'DOMAIN_ADMIN', 'WORKSPACE_OWNER', 'WORKSPACE_MEMBER')
    )
    api_permissions = ListField(StringField(max_length=255), default=[])
    page_permissions = ListField(EmbeddedDocumentField(PagePermission), default=[])
    tags = DictField(default=None)
    is_managed = BooleanField(default=False)
    domain_id = StringField(max_length=40)
    created_at = DateTimeField(auto_now_add=True)
    updated_at = DateTimeField(auto_now=True)

    meta = {
        'updatable_fields': [
            'name',
            'api_permissions',
            'page_permissions',
            'tags',
            'updated_at'
        ],
        'minimal_fields': [
            'role_id',
            'name',
            'role_type',
            'is_managed'
        ],
        'ordering': ['name'],
        'indexes': [
            'role_type',
            'domain_id'
        ]
    }


from typing import Union, Literal, List
from pydantic import BaseModel

__all__ = [
    "ProjectCreateRequest",
    "ProjectUpdateRequest",
    "ProjectUpdateProjectTypeRequest",
    "ProjectDeleteRequest",
    "ProjectAddUsersRequest",
    "ProjectRemoveUsersRequest",
    "ProjectAddUserGroupsRequest",
    "ProjectRemoveUserGroupsRequest",
    "ProjectGetRequest",
    "ProjectSearchQueryRequest",
    "ProjectStatQueryRequest",
    "ProjectChangeProjectGroupRequest",
    "ProjectType",
]

ProjectType = Literal["PRIVATE", "PUBLIC"]


class ProjectCreateRequest(BaseModel):
    name: str
    project_type: ProjectType
    tags: Union[dict, None] = None
    project_group_id: Union[str, None] = None
    workspace_id: str
    domain_id: str


class ProjectUpdateRequest(BaseModel):
    project_id: str
    name: Union[str, None] = None
    tags: Union[dict, None] = None
    workspace_id: str
    domain_id: str
    user_projects: Union[list, None] = None


class ProjectUpdateProjectTypeRequest(BaseModel):
    project_id: str
    project_type: ProjectType
    workspace_id: str
    domain_id: str


class ProjectChangeProjectGroupRequest(BaseModel):
    project_id: str
    project_group_id: Union[str, None] = None
    workspace_id: str
    domain_id: str


class ProjectDeleteRequest(BaseModel):
    project_id: str
    workspace_id: str
    domain_id: str


class ProjectAddUsersRequest(BaseModel):
    project_id: str
    users: List[str]
    workspace_id: str
    domain_id: str
    user_projects: Union[list, None] = None


class ProjectRemoveUsersRequest(BaseModel):
    project_id: str
    users: List[str]
    workspace_id: str
    domain_id: str
    user_projects: Union[list, None] = None


class ProjectAddUserGroupsRequest(BaseModel):
    project_id: str
    user_groups: List[str]
    workspace_id: str
    domain_id: str
    user_projects: Union[list, None] = None


class ProjectRemoveUserGroupsRequest(BaseModel):
    project_id: str
    user_groups: List[str]
    workspace_id: str
    domain_id: str
    user_projects: Union[list, None] = None


class ProjectGetRequest(BaseModel):
    project_id: str
    workspace_id: str
    domain_id: str
    user_projects: Union[list, None] = None


class ProjectSearchQueryRequest(BaseModel):
    query: Union[dict, None] = None
    project_id: Union[str, None] = None
    name: Union[str, None] = None
    project_type: Union[ProjectType, None] = None
    user_id: Union[str, None] = None
    user_group_id: Union[str, None] = None
    project_group_id: Union[str, None] = None
    workspace_id: Union[str, None] = None
    domain_id: str
    user_projects: Union[list, None] = None


class ProjectStatQueryRequest(BaseModel):
    query: dict
    workspace_id: Union[str, None] = None
    domain_id: str
    user_projects: Union[list, None] = None




import logging
from typing import Union

from spaceone.core.service import *
from spaceone.core.service.utils import *

from spaceone.identity.model.endpoint.request import *
from spaceone.identity.model.endpoint.response import *
from spaceone.identity.manager.endpoint_manager import EndpointManager

_LOGGER = logging.getLogger(__name__)


class EndpointService(BaseService):

    service = "identity"
    resource = "Endpoint"
    permission_group = "PUBLIC"

    @transaction(scope='public')
    # @append_query_filter(['service'])
    # @append_keyword_filter(['service'])
    @convert_model
    def list(
        self, params: EndpointSearchQueryRequest
    ) -> Union[EndpointsResponse, dict]:
        """list endpoints of service

        Args:
            params (EndpointSearchQueryRequest): {
                'query': 'dict (spaceone.api.core.v1.Query)',
                'service': 'str'
            }

        Returns:
            EndpointsResponse:
        """

        endpoint_mgr: EndpointManager = EndpointManager()
        endpoints_info, total_count = endpoint_mgr.list_endpoints(params.service)

        return EndpointsResponse(results=endpoints_info, total_count=total_count)



import logging
from datetime import datetime

from spaceone.identity.error.error_authentication import *
from spaceone.identity.error.error_user import ERROR_USER_STATUS_CHECK_FAILURE
from spaceone.identity.lib.cipher import PasswordCipher
from spaceone.identity.lib.key_generator import KeyGenerator
from spaceone.identity.manager.user_manager import UserManager
from spaceone.identity.manager.token_manager.base import JWTManager
from spaceone.identity.model.user.database import User

_LOGGER = logging.getLogger(__name__)


class LocalTokenManager(JWTManager):
    auth_type = "LOCAL"

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.user_mgr = UserManager()

    def authenticate(self, user_id, domain_id, credentials):
        pw_to_check = self._parse_password(credentials)

        self.user = self.user_mgr.get_user(user_id, domain_id)

        self._check_user_state()

        # TODO: decrypt pw
        is_correct = PasswordCipher().checkpw(pw_to_check, self.user.password)
        _LOGGER.debug(
            f"[authenticate] is_correct: {is_correct}, pw_to_check: {pw_to_check}, hashed_pw: {self.user.password}"
        )

        if is_correct:
            self.is_authenticated = True
        else:
            raise ERROR_AUTHENTICATION_FAILURE(user_id=self.user.user_id)

    def issue_temporary_token(self, user_id, domain_id, **kwargs):
        api_permissions = [
            "identity.User.get:user:read",
            "identity.User.update:user:write",
        ]
        private_jwk = self._get_private_jwk(kwargs)
        expired = kwargs["timeout"]

        key_gen = KeyGenerator(
            prv_jwk=private_jwk, domain_id=domain_id, audience=self.user.user_id
        )

        # Issue token
        access_token = key_gen.generate_access_token(
            expired=expired, api_permissions=api_permissions
        )

        return {"access_token": access_token}

    def issue_token(self, **kwargs):
        if self.is_authenticated is False:
            raise ERROR_NOT_AUTHENTICATED()

        api_permissions = self._get_permissions_from_required_actions()
        private_jwk = self._get_private_jwk(kwargs)
        refresh_private_jwk = self._get_refresh_private_jwk(kwargs)
        domain_id = kwargs.get("domain_id")

        key_gen = KeyGenerator(
            prv_jwk=private_jwk,
            domain_id=domain_id,
            audience=self.user.user_id,
            refresh_prv_jwk=refresh_private_jwk,
        )

        ttl = kwargs.get("ttl") or self.CONST_REFRESH_TTL
        timeout = kwargs.get("timeout") or self.CONST_TOKEN_TIMEOUT

        access_token = key_gen.generate_access_token(
            expired=timeout, api_permissions=api_permissions
        )

        refresh_token = key_gen.generate_refresh_token(expired=timeout, ttl=ttl)
        user = self.user.update({"last_accessed_at": datetime.utcnow()})

        return {"access_token": access_token, "refresh_token": refresh_token}

    def refresh_token(self, user_id, domain_id, **kwargs):
        self.user: User = self.user_mgr.get_user(user_id, domain_id)
        self._check_user_state()

        return self.issue_token(domain_id=domain_id, **kwargs)

    def _get_permissions_from_required_actions(self):
        if "UPDATE_PASSWORD" in self.user.required_actions:
            return ["identity.User.get:user:read", "identity.User.update:user:write"]

        return None

    @staticmethod
    def _parse_password(credentials):
        pw_to_check = credentials.get("password", None)

        if pw_to_check is None:
            raise ERROR_INVALID_CREDENTIALS()

        return pw_to_check

    def _check_user_state(self):
        if self.user.state != "ENABLED":
            raise ERROR_USER_STATUS_CHECK_FAILURE(user_id=self.user.user_id)

        if self.user.auth_type != "LOCAL":
            raise ERROR_NOT_FOUND(key="user_id", value=self.user.user_id)



import logging
from typing import Tuple

from mongoengine import QuerySet
from spaceone.core.manager import BaseManager
from spaceone.core import utils

from spaceone.identity.lib.key_generator import KeyGenerator
from spaceone.identity.model.app.database import App
from spaceone.identity.model.api_key.database import APIKey
from spaceone.identity.model.user.database import User
from spaceone.identity.manager.domain_secret_manager import DomainSecretManager

_LOGGER = logging.getLogger(__name__)


class APIKeyManager(BaseManager):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.api_key_model = APIKey

    def create_api_key(self, params: dict) -> Tuple[APIKey, str]:
        def _rollback(vo):
            _LOGGER.info(f"[create_api_key._rollback] Delete api key: {vo.api_key_id}")
            vo.delete()

        if params["owner_type"] == "USER":
            audience = params["user_id"]
        else:
            audience = params["app_id"]

        api_key_vo = self.api_key_model.create(params)
        self.transaction.add_rollback(_rollback, api_key_vo)

        domain_secret_mgr = DomainSecretManager()
        prv_jwk = domain_secret_mgr.get_domain_private_key(params["domain_id"])
        params["expired_at"] = utils.iso8601_to_datetime(
            params["expired_at"]
        ).timestamp()

        key_gen = KeyGenerator(
            api_key_id=api_key_vo.api_key_id,
            prv_jwk=prv_jwk,
            domain_id=params["domain_id"],
            audience=audience,
        )

        api_key = key_gen.generate_api_key(api_key_vo.api_key_id, params["expired_at"])

        return api_key_vo, api_key

    def create_api_key_by_user_vo(
        self, user_vo: User, params: dict
    ) -> Tuple[APIKey, str]:
        params["user"] = user_vo
        params["owner_type"] = "USER"
        return self.create_api_key(params)

    def create_api_key_by_app_vo(self, app_vo: App, params: dict) -> Tuple[APIKey, str]:
        params["app"] = app_vo
        params["app_id"] = app_vo.app_id
        params["owner_type"] = "APP"
        return self.create_api_key(params)

    def update_api_key_by_vo(self, params: dict, api_key_vo: APIKey) -> APIKey:
        def _rollback(old_data):
            _LOGGER.info(
                f"[update_api_key_by_vo._rollback] Revert Data : {old_data['api_key_id']}"
            )
            api_key_vo.update(old_data)

        self.transaction.add_rollback(_rollback, api_key_vo.to_dict())

        return api_key_vo.update(params)

    @staticmethod
    def delete_api_key_by_vo(api_key_vo: APIKey) -> None:
        api_key_vo.delete()

    def enable_api_key(self, api_key_vo: APIKey) -> APIKey:
        return self.update_api_key_by_vo({"state": "ENABLED"}, api_key_vo)

    def disable_api_key(self, api_key_vo: APIKey) -> APIKey:
        return self.update_api_key_by_vo({"state": "DISABLED"}, api_key_vo)

    def get_api_key(
        self,
        api_key_id: str,
        domain_id: str,
        owner_type: str,
        user_id: str = None,
        app_id: str = None,
    ) -> APIKey:
        conditions = {
            "api_key_id": api_key_id,
            "domain_id": domain_id,
            "owner_type": owner_type,
        }

        if user_id:
            conditions["user_id"] = user_id

        if app_id:
            conditions["app_id"] = app_id

        return self.api_key_model.get(**conditions)

    def filter_api_keys(self, **conditions) -> QuerySet:
        return self.api_key_model.filter(**conditions)

    def list_api_keys(self, query: dict) -> Tuple[list, int]:
        return self.api_key_model.query(**query)

    def stat_api_keys(self, query: dict) -> dict:
        return self.api_key_model.stat(**query)



from spaceone.core.error import *


class ERROR_RELATED_SERVICE_ACCOUNT_EXIST(ERROR_INVALID_ARGUMENT):
    _message = "Related service account is exist. (service_account_id = {service_account_id})"


class ERROR_NOT_ALLOWED_ADD_USER_TO_PUBLIC_PROJECT(ERROR_INVALID_ARGUMENT):
    _message = "Not allowed to add user to public project."



from typing import Union
from pydantic import BaseModel

__all__ = [
    "WorkspaceCreateRequest",
    "WorkspaceUpdateRequest",
    "WorkspaceDeleteRequest",
    "WorkspaceEnableRequest",
    "WorkspaceDisableRequest",
    "WorkspaceGetRequest",
    "WorkspaceSearchQueryRequest",
    "WorkspaceStatQueryRequest",
]


class WorkspaceCreateRequest(BaseModel):
    name: str
    tags: Union[dict, None] = None
    domain_id: str


class WorkspaceUpdateRequest(BaseModel):
    workspace_id: str
    name: Union[str, None] = None
    tags: Union[dict, None] = None
    domain_id: str


class WorkspaceDeleteRequest(BaseModel):
    workspace_id: str
    domain_id: str


class WorkspaceEnableRequest(BaseModel):
    workspace_id: str
    domain_id: str


class WorkspaceDisableRequest(BaseModel):
    workspace_id: str
    domain_id: str


class WorkspaceGetRequest(BaseModel):
    workspace_id: str
    domain_id: str


class WorkspaceSearchQueryRequest(BaseModel):
    query: Union[dict, None] = None
    name: Union[str, None] = None
    workspace_id: Union[str, None] = None
    domain_id: str


class WorkspaceStatQueryRequest(BaseModel):
    query: dict
    workspace_id: Union[str, None] = None
    domain_id: str



from mongoengine import *
from spaceone.core.model.mongo_model import MongoModel


class TrustedAccount(MongoModel):
    trusted_account_id = StringField(max_length=40, generate_id='ta', unique=True)
    name = StringField(max_length=255, unique_with=['domain_id'])
    data = DictField(default=None)
    provider = StringField(max_length=40)
    tags = DictField(default=None)
    permission_group = StringField(max_length=40, choices=('DOMAIN', 'WORKSPACE'))
    workspace_id = StringField(max_length=40)
    domain_id = StringField(max_length=40)
    created_at = DateTimeField(auto_now_add=True)

    meta = {
        'updatable_fields': [
            'name',
            'data',
            'tags'
        ],
        'minimal_fields': [
            'trusted_account_id',
            'name',
            'provider',
            'permission_group',
            'workspace_id'
        ],
        'ordering': ['name'],
        'indexes': [
            'provider',
            'permission_group',
            'workspace_id',
            'domain_id'
        ]
    }



from spaceone.core.error import *


class ERROR_USER_STATUS_CHECK_FAILURE(ERROR_BASE):
    _message = 'A user "{user_id}" status is not ENABLED.'


class ERROR_EXTERNAL_USER_NOT_ALLOWED_API_USER(ERROR_INVALID_ARGUMENT):
    _message = "External user cannot be created with the API_USER type."


class ERROR_NOT_ALLOWED_EXTERNAL_AUTHENTICATION(ERROR_INVALID_ARGUMENT):
    _message = "This domain does not allow external authentication."


class ERROR_TOO_MANY_USERS_IN_EXTERNAL_AUTH(ERROR_INVALID_ARGUMENT):
    _message = "There are two or more users in the external authentication system. (user_id = {user_id})"


class ERROR_NOT_FOUND_USER_IN_EXTERNAL_AUTH(ERROR_INVALID_ARGUMENT):
    _message = "The user could not be found in the external authentication system. (user_id = {user_id})"


class ERROR_INCORRECT_PASSWORD_FORMAT(ERROR_INVALID_ARGUMENT):
    _message = "The password format is incorrect. (rule = {rule})"


class ERROR_INCORRECT_USER_ID_FORMAT(ERROR_INVALID_ARGUMENT):
    _message = "The user id format is incorrect. (rule = {rule})"


class ERROR_NOT_ALLOWED_ACTIONS(ERROR_INVALID_ARGUMENT):
    _message = "External or API user are supported for actions. (action = {action})"


class ERROR_VERIFICATION_UNAVAILABLE(ERROR_INVALID_ARGUMENT):
    _message = "Email verification incomplete. (user_id = {user_id})"


class ERROR_INVALID_VERIFY_CODE(ERROR_VERIFICATION_UNAVAILABLE):
    _message = "Invalid verify code. (verify_code = {verify_code})"


class ERROR_UNABLE_TO_RESET_PASSWORD(ERROR_INVALID_ARGUMENT):
    _message = "Unable to reset password. (user_id = {user_id})"


class ERROR_PASSWORD_NOT_CHANGED(ERROR_UNABLE_TO_RESET_PASSWORD):
    _message = (
        "The password cannot be the same as the old password. (user_id = {user_id})"
    )


class ERROR_UNABLE_TO_RESET_PASSWORD_IN_EXTERNAL_AUTH(ERROR_UNABLE_TO_RESET_PASSWORD):
    _message = "Unable to reset password in external authentication system. (user_id = {user_id})"


class ERROR_UNABLE_TO_RESET_PASSWORD_WITHOUT_EMAIL(ERROR_UNABLE_TO_RESET_PASSWORD):
    _message = "Unable to reset password without email. (user_id = {user_id})"


class ERROR_LAST_ADMIN_CANNOT_DISABLE(ERROR_INVALID_ARGUMENT):
    _message = "The last admin cannot be disabled. (user_id = {user_id})"



import logging
from typing import Tuple, List

from spaceone.core.manager import BaseManager
from spaceone.core.connector.space_connector import SpaceConnector
from spaceone.identity.model.service_account.database import ServiceAccount

_LOGGER = logging.getLogger(__name__)


class ServiceAccountManager(BaseManager):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.service_account_model = ServiceAccount

    def create_service_account(self, params: dict) -> ServiceAccount:
        def _rollback(vo: ServiceAccount):
            _LOGGER.info(f'[create_service_account._rollback] '
                         f'Delete service account: {vo.name} ({vo.service_account_id})')
            service_account_vo.delete()

        service_account_vo = self.service_account_model.create(params)
        self.transaction.add_rollback(_rollback, service_account_vo)

        return service_account_vo

    def update_service_account(self, params: dict) -> ServiceAccount:
        service_account_vo = self.get_service_account(
            params['service_account_id'],
            params['domain_id'],
            params['workspace_id'],
            params.get('user_projects')
        )

        return self.update_service_account_by_vo(params, service_account_vo)

    def update_service_account_by_vo(self, params: dict, service_account_vo: ServiceAccount) -> ServiceAccount:
        def _rollback(old_data):
            _LOGGER.info(f'[update_service_account_by_vo._rollback] Revert Data : '
                         f'{old_data["service_account_id"]}')
            service_account_vo.update(old_data)

        self.transaction.add_rollback(_rollback, service_account_vo.to_dict())

        return service_account_vo.update(params)

    @staticmethod
    def delete_service_account_by_vo(service_account_vo: ServiceAccount) -> None:
        service_account_vo.delete()

    def get_service_account(
        self, service_account_id: str, domain_id: str, workspace_id: str, user_projects: List[str] = None,
    ) -> ServiceAccount:

        conditions = {
            'service_account_id': service_account_id,
            'domain_id': domain_id,
            'workspace_id': workspace_id,
        }

        if user_projects:
            conditions['project_id'] = user_projects

        return self.service_account_model.get(**conditions)

    def filter_service_accounts(self, **conditions) -> List[ServiceAccount]:
        return self.service_account_model.filter(**conditions)

    def list_service_accounts(self, query: dict) -> Tuple[list, int]:
        return self.service_account_model.query(**query)

    def stat_service_accounts(self, query: dict) -> dict:
        return self.service_account_model.stat(**query)

    def update_secret_project(
        self, service_account_id: str, domain_id: str, workspace_id: str, project_id: str
    ) -> None:
        secret_connector = SpaceConnector(service='secret')
        response = self._list_secrets(secret_connector, service_account_id, domain_id, workspace_id)

        for secret_info in response.get('results', []):
            secret_connector.dispatch('Secret.update', {
                'secret_id': secret_info['secret_id'],
                'project_id': project_id,
                'domain_id': domain_id,
                'workspace_id': workspace_id
            })

    def delete_secrets(
        self, service_account_id: str, domain_id: str, workspace_id: str
    ) -> None:
        secret_connector = SpaceConnector(service='secret')
        response = self._list_secrets(secret_connector, service_account_id, domain_id, workspace_id)

        for secret_info in response.get('results', []):
            secret_connector.dispatch('Secret.delete', {
                'secret_id': secret_info['secret_id'],
                'domain_id': domain_id,
                'workspace_id': workspace_id
            })

    @staticmethod
    def get_all_service_account_ids_using_secret(domain_id: str, workspace_id: str) -> List[str]:
        secret_connector = SpaceConnector(service='secret')
        response = secret_connector.dispatch('Secret.stat', {
            'query': {
                'distinct': 'service_account_id',
                'filter': [
                    {
                        'k': 'service_account_id',
                        'v': None,
                        'o': 'not'
                    }
                ]
            },
            'domain_id': domain_id,
            'workspace_id': workspace_id
        })

        return response.get('results', [])

    @staticmethod
    def _list_secrets(
            secret_connector: SpaceConnector, service_account_id: str, domain_id: str, workspace_id: str
    ) -> dict:
        return secret_connector.dispatch('Secret.list', {
            'service_account_id': service_account_id,
            'domain_id': domain_id,
            'workspace_id': workspace_id
        })


from spaceone.core.error import *


class ERROR_USER_NOT_EXIST_IN_WORKSPACE(ERROR_INVALID_ARGUMENT):
    _message = "User does not exist in workspace. (user_id = {user_id}, workspace_id = {workspace_id})"



import logging
import time
import random
from abc import abstractmethod, ABC, ABCMeta

from spaceone.core import config, utils, cache
from spaceone.core.manager import BaseManager
from spaceone.core.auth.jwt.jwt_util import JWTUtil

from spaceone.identity.lib.key_generator import KeyGenerator
from spaceone.identity.error.error_token import *
from spaceone.identity.error.error_authentication import *

__all__ = ["TokenManager", "JWTManager"]
_LOGGER = logging.getLogger(__name__)


class TokenManager(BaseManager, ABC):
    is_authenticated = False

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._load_conf()

    @abstractmethod
    def issue_temporary_token(self, **kwargs):
        pass

    @abstractmethod
    def issue_token(self, **kwargs):
        pass

    @abstractmethod
    def refresh_token(self, user_id, domain_id, **kwargs):
        pass

    @abstractmethod
    def authenticate(self, user_id, domain_id, credentials):
        pass

    @abstractmethod
    def check_refreshable(self, key, ttl):
        pass

    def _load_conf(self):
        identity_conf = config.get_global("IDENTITY") or {}
        token_conf = identity_conf.get("token", {})
        self.CONST_TOKEN_TIMEOUT = token_conf.get("token_timeout", 1800)
        self.CONST_VERIFY_CODE_TIMEOUT = token_conf.get("verify_code_timeout", 3600)
        self.CONST_REFRESH_TIMEOUT = token_conf.get("refresh_timeout", 3600)
        self.CONST_REFRESH_TTL = token_conf.get("refresh_ttl", -1)
        self.CONST_REFRESH_ONCE = token_conf.get("refresh_once", True)


class JWTManager(TokenManager, metaclass=ABCMeta):
    auth_type = None
    user = None

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.old_refresh_key = None

    def issue_temporary_token(self, **kwargs):
        raise NotImplementedError("TokenManager.issue_temporary_token not implemented!")

    def issue_token(self, **kwargs):
        raise NotImplementedError("TokenManager.issue_token not implemented!")

    def refresh_token(self, user_id, domain_id, **kwargs):
        raise NotImplementedError("TokenManager.refresh_token not implemented!")

    def authenticate(self, user_id, domain_id, credentials):
        raise NotImplementedError("TokenManager.authenticate not implemented!")

    def check_refreshable(self, refresh_key, ttl):
        if self.CONST_REFRESH_ONCE:
            if (
                cache.is_set()
                and cache.get(f"identity:refresh-token:{refresh_key}") is None
            ):
                raise ERROR_INVALID_REFRESH_TOKEN()

        if ttl == 0:
            raise ERROR_REFRESH_COUNT()

        self.is_authenticated = True
        self.old_refresh_key = refresh_key

    def issue_access_token(self, user_id, domain_id, **kwargs):
        private_jwk = self._get_private_jwk(kwargs)
        permissions = kwargs.get("permissions")
        timeout = kwargs.get("timeout")
        if timeout is None:
            timeout = self.CONST_TOKEN_TIMEOUT

        payload = {
            "cat": "ACCESS_TOKEN",  # ACCESS_TOKEN | API_KEY
            # "token_type": "USER",  # USER | APP
            "did": domain_id,
            "aud": user_id,
            "iat": int(time.time()),
            "exp": int(time.time() + timeout),
        }

        if permissions:
            payload["permissions"] = permissions

        encoded = JWTUtil.encode(payload, private_jwk)
        return encoded

    def issue_refresh_token(self, user_type, user_id, domain_id, **kwargs):
        refresh_private_jwk = self._get_refresh_private_jwk(kwargs)
        ttl = kwargs.get("ttl")
        timeout = kwargs.get("timeout")

        if ttl is None:
            ttl = self.CONST_REFRESH_TTL
        elif ttl > self.CONST_REFRESH_TTL:
            raise ERROR_MAX_REFRESH_COUNT(max_refresh_count=self.CONST_REFRESH_TTL)

        if timeout is None:
            timeout = self.CONST_REFRESH_TIMEOUT

        refresh_key = self._generate_refresh_key()

        payload = {
            "cat": "REFRESH_TOKEN",
            "user_type": user_type,
            "did": domain_id,
            "aud": user_id,
            "iat": int(time.time()),
            "exp": int(time.time() + timeout),
            "key": refresh_key,
            "ttl": ttl,
        }

        encoded = JWTUtil.encode(payload, refresh_private_jwk)

        if self.CONST_REFRESH_ONCE:
            self._set_refresh_token_cache(refresh_key)

        return encoded

    @staticmethod
    def _generate_refresh_key():
        return utils.random_string(16)

    @staticmethod
    def _get_private_jwk(kwargs):
        if "private_jwk" not in kwargs:
            raise ERROR_NOT_FOUND_PRIVATE_KEY(purpose="Access Token")

        return kwargs["private_jwk"]

    @staticmethod
    def _get_refresh_private_jwk(kwargs):
        if "refresh_private_jwk" not in kwargs:
            raise ERROR_NOT_FOUND_PRIVATE_KEY(purpose="Refresh Token")

        return kwargs["refresh_private_jwk"]

    def _set_refresh_token_cache(self, new_refresh_key):
        if cache.is_set():
            if self.old_refresh_key:
                cache.delete(f"identity:refresh-token:{self.old_refresh_key}")

            cache.set(
                f"identity:refresh-token:{new_refresh_key}",
                "",
                expire=self.CONST_REFRESH_TIMEOUT,
            )

    def create_verify_code(self, user_id, domain_id):
        if cache.is_set():
            verify_code = self._generate_verify_code()
            cache.delete(f"verify-code:{domain_id}:{user_id}")
            cache.set(
                f"verify-code:{domain_id}:{user_id}",
                verify_code,
                expire=self.CONST_VERIFY_CODE_TIMEOUT,
            )
            return verify_code

    @staticmethod
    def check_verify_code(user_id, domain_id, verify_code):
        if cache.is_set():
            cached_verify_code = cache.get(f"verify-code:{domain_id}:{user_id}")
            if cached_verify_code == verify_code:
                return True
        return False

    @staticmethod
    def _generate_verify_code():
        return str(random.randint(100000, 999999))

    @classmethod
    def get_token_manager_by_auth_type(cls, auth_type):
        for subclass in cls.__subclasses__():
            if subclass.auth_type == auth_type:
                return subclass()
        raise ERROR_INVALID_AUTHENTICATION_TYPE(auth_type=auth_type)



import logging
from typing import Union
from spaceone.core import config

from spaceone.core.service import *
from spaceone.core.service.utils import *
from spaceone.core.error import *

from spaceone.identity.model.role.request import *
from spaceone.identity.model.role.response import *
from spaceone.identity.manager.role_manager import RoleManager
from spaceone.identity.manager.domain_manager import DomainManager

_LOGGER = logging.getLogger(__name__)


class RoleService(BaseService):

    service = "identity"
    resource = "Role"
    permission_group = "DOMAIN"

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.role_mgr = RoleManager()

    @transaction(scope="domain_admin:write")
    @convert_model
    def create(self, params: RoleCreateRequest) -> Union[RoleResponse, dict]:
        """ create role

         Args:
            params (RoleCreateRequest): {
                'name': 'str',                          # required
                'role_type': 'list',                    # required
                'api_permissions': 'list',              # required
                'page_permissions': 'list',
                'tags': 'dict',
                'domain_id': 'str'                      # required
            }

        Returns:
            RoleResponse:
        """

        if params.role_type in ['SYSTEM', 'SYSTEM_ADMIN']:
            domain_mgr = DomainManager()
            domain_vo = domain_mgr.get_domain(params.domain_id)

            root_domain_name = config.get_global('ROOT_DOMAIN_NAME', 'root')
            if domain_vo.name != root_domain_name:
                raise ERROR_PERMISSION_DENIED()

        role_vo = self.role_mgr.create_role(params.dict())
        return RoleResponse(**role_vo.to_dict())

    @transaction(scope="domain_admin:write")
    @convert_model
    def update(self, params: RoleUpdateRequest) -> Union[RoleResponse, dict]:
        """ update role

         Args:
            params (RoleUpdateRequest): {
                'role_id': 'str',                       # required
                'name': 'str',
                'api_permissions': 'list',
                'page_permissions': 'list',
                'tags': 'dict',
                'domain_id': 'str'                      # required
            }

        Returns:
            RoleResponse:
        """

        role_vo = self.role_mgr.get_role(params.role_id, params.domain_id)

        if params.api_permissions:
            # Check API Permissions
            pass

        role_vo = self.role_mgr.update_role_by_vo(
            params.dict(exclude_unset=True), role_vo
        )

        return RoleResponse(**role_vo.to_dict())

    @transaction(scope="domain_admin:write")
    @convert_model
    def delete(self, params: RoleDeleteRequest) -> None:
        """ delete role

         Args:
            params (RoleDeleteRequest): {
                'role_id': 'str',               # required
                'domain_id': 'str',             # required
            }

        Returns:
            None
        """

        role_vo = self.role_mgr.get_role(params.role_id, params.domain_id)
        self.role_mgr.delete_role_by_vo(role_vo)

    @transaction(scope="workspace_member:read")
    @convert_model
    def get(self, params: RoleGetRequest) -> Union[RoleResponse, dict]:
        """ get role

         Args:
            params (RoleGetRequest): {
                'role_id': 'str',               # required
                'domain_id': 'str',             # required
            }

        Returns:
             RoleResponse:
        """

        role_vo = self.role_mgr.get_role(params.role_id, params.domain_id)
        return RoleResponse(**role_vo.to_dict())

    @transaction(scope="workspace_member:read")
    @append_query_filter(['role_id', 'role_type', 'domain_id'])
    @append_keyword_filter(['role_id', 'name'])
    @convert_model
    def list(self, params: RoleSearchQueryRequest) -> Union[RolesResponse, dict]:
        """ list roles

        Args:
            params (RoleSearchQueryRequest): {
                'query': 'dict (spaceone.api.core.v1.Query)',
                'role_id': 'str',
                'role_type': 'str',
                'domain_id': 'str',                     # required
            }

        Returns:
            RolesResponse:
        """

        query = params.query or {}
        role_vos, total_count = self.role_mgr.list_roles(query, params.domain_id)

        roles_info = [role_vo.to_dict() for role_vo in role_vos]
        return RolesResponse(results=roles_info, total_count=total_count)

    @transaction(scope="workspace_member:read")
    @append_query_filter(['domain_id'])
    @append_keyword_filter(['role_id', 'name'])
    @convert_model
    def stat(self, params: RoleStatQueryRequest) -> dict:
        """ stat roles

        Args:
            params (PolicyStatQueryRequest): {
                'query': 'dict (spaceone.api.core.v1.StatisticsQuery)', # required
                'domain_id': 'str',         # required
            }

        Returns:
            dict: {
                'results': 'list',
                'total_count': 'int'
            }
        """

        query = params.query or {}
        return self.role_mgr.stat_roles(query)



import logging
from spaceone.core.service import BaseService, transaction, convert_model
from spaceone.identity.model.authorization.request import *
from spaceone.identity.model.authorization.response import *

_LOGGER = logging.getLogger(__name__)


class AuthorizationService(BaseService):
    @transaction
    @convert_model
    def verify(
        self, params: AuthorizationVerifyRequest
    ) -> Union[AuthorizationResponse, dict]:
        return {}



from spaceone.core.error import *

class ERROR_POLICY_IN_USED(ERROR_INVALID_ARGUMENT):
    _message = 'Policy is used. (role_id = {role_id})'


class ERROR_ROLE_IN_USED(ERROR_INVALID_ARGUMENT):
    _message = 'Role is used. (role_binding_id = {role_binding_id}, user_id = {user_id})'


class ERROR_NOT_ALLOWED_ROLE_TYPE(ERROR_INVALID_ARGUMENT):
    _message = 'Role type is not allowed. (supported_role_type = {supported_role_type})'



from datetime import datetime
from typing import Union, List
from pydantic import BaseModel

from spaceone.core import utils

from spaceone.identity.model.project.request import ProjectType

__all__ = ["ProjectResponse", "ProjectsResponse"]


class ProjectResponse(BaseModel):
    project_id: Union[str, None] = None
    name: Union[str, None] = None
    project_type: Union[ProjectType, None] = None
    tags: Union[dict, None] = None
    users: Union[List[str], None] = None
    user_groups: Union[List[str], None] = None
    project_group_id: Union[str, None] = None
    workspace_id: Union[str, None] = None
    domain_id: Union[str, None] = None
    created_at: Union[datetime, None] = None

    def dict(self, *args, **kwargs):
        data = super().dict(*args, **kwargs)
        data["created_at"] = utils.datetime_to_iso8601(data["created_at"])
        return data


class ProjectsResponse(BaseModel):
    results: List[ProjectResponse] = []
    total_count: int



import logging
from typing import Tuple
from mongoengine import QuerySet
from spaceone.core.manager import BaseManager

from spaceone.identity.error.error_project_group import *
from spaceone.identity.manager.project_manager import ProjectManager
from spaceone.identity.model.project_group.database import ProjectGroup

_LOGGER = logging.getLogger(__name__)


class ProjectGroupManager(BaseManager):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.project_group_model = ProjectGroup

    def create_project_group(self, params: dict) -> ProjectGroup:
        def _rollback(vo: ProjectGroup):
            _LOGGER.info(
                f"[create_project_group._rollback] Delete project group: {vo.name} ({vo.project_group_id})"
            )
            vo.delete()

        project_group_vo = self.project_group_model.create(params)
        self.transaction.add_rollback(_rollback, project_group_vo)

        return project_group_vo

    def update_project_group_by_vo(
        self, params: dict, project_group_vo: ProjectGroup
    ) -> ProjectGroup:
        def _rollback(old_data):
            _LOGGER.info(
                f'[update_project_group._rollback] Revert Data: {old_data["name"]} ({old_data["project_group_id"]})'
            )
            project_group_vo.update(old_data)

        self.transaction.add_rollback(_rollback, project_group_vo.to_dict())

        return project_group_vo.update(params)

    def delete_project_group_by_vo(self, project_group_vo: ProjectGroup) -> None:
        project_mgr = ProjectManager()
        project_vos = project_mgr.filter_projects(
            project_group_id=project_group_vo.project_group_id
        )
        for project_vo in project_vos:
            raise ERROR_RELATED_PROJECT_EXIST(project_id=project_vo.project_id)

        child_vos = self.filter_project_groups(
            parent_group_id=project_group_vo.project_group_id
        )
        for child_vo in child_vos:
            raise ERROR_RELATED_PROJECT_GROUP_EXIST(
                project_group_id=child_vo.project_group_id
            )

        project_group_vo.delete()

    def get_project_group(
        self, project_group_id: str, workspace_id: str, domain_id: str
    ) -> ProjectGroup:
        conditions = {
            "project_group_id": project_group_id,
            "workspace_id": workspace_id,
            "domain_id": domain_id,
        }

        return self.project_group_model.get(**conditions)

    def filter_project_groups(self, **conditions) -> QuerySet:
        return self.project_group_model.filter(**conditions)

    def list_project_groups(self, query: dict) -> Tuple[QuerySet, int]:
        return self.project_group_model.query(**query)

    def stat_project_groups(self, query: dict) -> dict:
        return self.project_group_model.stat(**query)



from spaceone.core.error import *


class ERROR_AUTHENTICATION_FAILURE(ERROR_AUTHENTICATE_FAILURE):
    _message = 'A user "{user_id}" is not exist or password is not correct.'


class ERROR_NOT_AUTHENTICATED(ERROR_AUTHENTICATE_FAILURE):
    _message = "A user is not authenticated."


class ERROR_NOT_ALLOWED_ISSUE_TOKEN_API_USER(ERROR_AUTHENTICATE_FAILURE):
    _message = "API_USER are not allowed to issue tokens. (user_id = {user_id})"


class ERROR_INVALID_CREDENTIALS(ERROR_AUTHENTICATE_FAILURE):
    _message = "Invalid credentials provided."


class ERROR_AUTHENTICATION_FAILURE_PLUGIN(ERROR_INTERNAL_API):
    _message = "External plugin authentication exception. (reason = {message})"


class ERROR_INVALID_REFRESH_TOKEN(ERROR_AUTHENTICATE_FAILURE):
    _message = "Refresh token is invalid or expired."


class ERROR_REFRESH_COUNT(ERROR_AUTHENTICATE_FAILURE):
    _message = "The refresh count is exhausted."


class ERROR_NOT_FOUND_PRIVATE_KEY(ERROR_AUTHENTICATE_FAILURE):
    _message = "Private key not found. (purpose = {purpose})"


class ERROR_MAX_REFRESH_COUNT(ERROR_AUTHENTICATE_FAILURE):
    _message = "The maximum refresh count has been exceeded. (max_refresh_count = {max_refresh_count})"



import logging
from typing import Tuple

from mongoengine import QuerySet
from spaceone.core.connector.space_connector import SpaceConnector
from spaceone.core.manager import BaseManager

from spaceone.identity.connector.auth_plugin_connector import AuthPluginConnector
from spaceone.identity.model.domain.database import Domain
from spaceone.identity.model.external_auth.database import ExternalAuth

_LOGGER = logging.getLogger(__name__)


class ExternalAuthManager(BaseManager):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.external_auth_model = ExternalAuth

    def set_external_auth(self, params: dict, domain_vo: Domain) -> ExternalAuth:
        def _rollback(old_data):
            _LOGGER.info(
                f'[set_external_auth._rollback] Revert Data: {old_data["domain_id"]}'
            )
            external_auth_vo.update(old_data)

        if plugin_info := params.get("plugin_info"):
            domain_id = domain_vo.domain_id
            options = plugin_info.get("options", {})
            endpoint, updated_version = self.get_auth_plugin_endpoint(
                domain_id, plugin_info
            )

            if updated_version:
                params["plugin_info"]["version"] = updated_version

            response = self.init_auth_plugin(endpoint, options)
            params["plugin_info"]["metadata"] = response["metadata"]

            secret_data = plugin_info.get("secret_data")

            if secret_data:
                schema = plugin_info.get("schema")
                secret_id = self._create_secret(domain_id, secret_data, schema)

                if secret_id:
                    params["plugin_info"]["secret_id"] = secret_id
                    del params["plugin_info"]["secret_data"]

        params.update({"state": "ENABLED"})
        external_auth_vo = self.external_auth_model.create(params)
        self.transaction.add_rollback(_rollback, external_auth_vo.to_dict())

        return external_auth_vo

    def get_external_auth(self, domain_id: str) -> ExternalAuth:
        return self.external_auth_model.get(domain_id=domain_id)

    def get_auth_info(self, domain_vo: Domain) -> dict:
        external_auth_vos = self.filter_external_auth(domain_id=domain_vo.domain_id)

        if external_auth_vos.count() > 0:
            external_auth_state = "ENABLED"
            metadata = (external_auth_vos[0].plugin_info.get("metadata", {}),)

        else:
            external_auth_state = "DISABLED"
            metadata = {}

        return {
            "domain_id": domain_vo.domain_id,
            "name": domain_vo.name,
            "external_auth_state": external_auth_state,
            "metadata": metadata,
        }

    def filter_external_auth(self, **conditions) -> QuerySet:
        return self.external_auth_model.filter(**conditions)

    @staticmethod
    def get_auth_plugin_endpoint(domain_id: str, plugin_info: dict) -> Tuple[str, str]:
        plugin_connector = SpaceConnector(service="plugin")
        response = plugin_connector.dispatch(
            "Plugin.get_plugin_endpoint",
            {
                "plugin_id": plugin_info["plugin_id"],
                "version": plugin_info.get("version"),
                "upgrade_mode": plugin_info.get("upgrade_mode", "AUTO"),
                "domain_id": domain_id,
            },
        )

        return response["endpoint"], response.get("updated_version")

    @staticmethod
    def init_auth_plugin(endpoint: str, options: dict) -> dict:
        auth_conn = AuthPluginConnector()
        auth_conn.initialize(endpoint)

        return auth_conn.init(options)

    @staticmethod
    def _create_secret(domain_id: str, secret_data: dict, schema: dict) -> str:
        secret_connector = SpaceConnector(service="secret")

        params = {
            "name": f"{domain_id}-auth-plugin-credentials",
            "data": secret_data,
            "secret_type": "CREDENTIALS",
            "schema": schema,
            "domain_id": domain_id,
        }

        resp = secret_connector.dispatch("Secret.create", params)
        _LOGGER.debug(f"[_create_secret] {resp}")
        return resp.get("secret_id")



import logging
from typing import Union

from spaceone.core.service import *
from spaceone.core.service.utils import *

from spaceone.identity.model.service_account.request import *
from spaceone.identity.model.service_account.response import *
from spaceone.identity.manager.schema_manager import SchemaManager
from spaceone.identity.manager.service_account_manager import ServiceAccountManager
from spaceone.identity.manager.trusted_account_manager import TrustedAccountManager


_LOGGER = logging.getLogger(__name__)


class ServiceAccountService(BaseService):

    service = "identity"
    resource = "ServiceAccount"
    permission_group = "PROJECT"

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.service_account_mgr = ServiceAccountManager()

    @transaction(scope="workspace_member:write")
    @convert_model
    def create(self, params: ServiceAccountCreateRequest) -> Union[ServiceAccountResponse, dict]:
        """ create service account

         Args:
            params (ServiceAccountCreateRequest): {
                'name': 'str',                          # required
                'data': 'dict',                         # required
                'provider': 'str',                      # required
                'trusted_account_id': 'str',
                'tags': 'dict',
                'project_id': 'str',                    # required
                'workspace_id': 'str',                  # required
                'domain_id': 'str'                      # required
            }

        Returns:
            ServiceAccountResponse:
        """

        # Check data by schema
        schema_mgr = SchemaManager()
        schema_mgr.validate_data_by_schema(
            params.provider, params.domain_id, 'SERVICE_ACCOUNT', params.data
        )

        # Check trusted service account
        if params.trusted_account_id:
            trusted_account_mgr = TrustedAccountManager()
            trusted_account_mgr.get_trusted_account(
                params.trusted_account_id, params.domain_id, params.workspace_id
            )

        service_account_vo = self.service_account_mgr.create_service_account(params.dict())
        return ServiceAccountResponse(**service_account_vo.to_dict())

    @transaction(scope="workspace_member:write")
    @convert_model
    def update(self, params: ServiceAccountUpdateRequest) -> Union[ServiceAccountResponse, dict]:
        """ update service account

         Args:
            params (ServiceAccountUpdateRequest): {
                'service_account_id': 'str',        # required
                'name': 'str',
                'data': 'dict',
                'tags': 'dict',
                'project_id': 'str',
                'workspace_id': 'str',              # required
                'domain_id': 'str',                 # required
                'user_projects': 'list'             # from meta
            }

        Returns:
            ServiceAccountResponse:
        """

        service_account_vo = self.service_account_mgr.get_service_account(
            params.service_account_id, params.domain_id, params.workspace_id, params.user_projects
        )

        if params.data:
            # Check data by schema
            schema_mgr = SchemaManager()
            schema_mgr.validate_data_by_schema(
                service_account_vo.provider, params.domain_id, 'SERVICE_ACCOUNT', params.data
            )

        service_account_vo = self.service_account_mgr.update_service_account_by_vo(
            params.dict(exclude_unset=True), service_account_vo
        )

        return ServiceAccountResponse(**service_account_vo.to_dict())

    @transaction(scope="workspace_member:write")
    @convert_model
    def change_trusted_account(
        self, params: ServiceAccountChangeTrustedAccountRequest
    ) -> Union[ServiceAccountResponse, dict]:
        """ change trusted service account

         Args:
            params (ServiceAccountChangeTrustedAccountRequest): {
                'service_account_id': 'str',            # required
                'trusted_account_id': 'str',            # required
                'workspace_id': 'str',                  # required
                'domain_id': 'str',                     # required
                'user_projects': 'list'                 # from meta
            }

        Returns:

            ServiceAccountResponse:
        """

        service_account_vo = self.service_account_mgr.get_service_account(
            params.service_account_id, params.domain_id, params.workspace_id, params.user_projects
        )

        # Check trusted service account
        trusted_account_mgr = TrustedAccountManager()
        trusted_account_mgr.get_trusted_account(
            params.trusted_account_id, params.domain_id, params.workspace_id
        )

        service_account_vo = self.service_account_mgr.update_service_account_by_vo(
            {'trusted_account_id': params.trusted_account_id}, service_account_vo
        )

        return ServiceAccountResponse(**service_account_vo.to_dict())

    @transaction(scope="workspace_member:write")
    @convert_model
    def delete(self, params: ServiceAccountDeleteRequest) -> None:
        """ delete service account

         Args:
            params (ServiceAccountDeleteRequest): {
                'service_account_id': 'str',            # required
                'workspace_id': 'str',                  # required
                'domain_id': 'str',                     # required
                'user_projects': 'list'                 # from meta
            }

        Returns:
            None
        """

        service_account_vo = self.service_account_mgr.get_service_account(
            params.service_account_id, params.domain_id, params.workspace_id, params.user_projects
        )

        self.service_account_mgr.delete_service_account_by_vo(service_account_vo)

    @transaction(scope="workspace_member:read")
    @convert_model
    def get(self, params: ServiceAccountGetRequest) -> Union[ServiceAccountResponse, dict]:
        """ get service account

         Args:
            params (ServiceAccountDeleteRequest): {
                'service_account_id': 'str',            # required
                'workspace_id': 'str',                  # required
                'domain_id': 'str',                     # required
                'user_projects': 'list'                 # from meta
            }

        Returns:
             ServiceAccountResponse:
        """

        service_account_vo = self.service_account_mgr.get_service_account(
            params.service_account_id, params.domain_id, params.workspace_id, params.user_projects
        )

        return ServiceAccountResponse(**service_account_vo.to_dict())

    @transaction(scope="workspace_member:read")
    @append_query_filter([
        'service_account_id', 'name', 'provider', 'project_id', 'workspace_id', 'domain_id', 'user_projects'
    ])
    @append_keyword_filter(['service_account_id', 'name'])
    @set_query_page_limit(1000)
    @convert_model
    def list(self, params: ServiceAccountSearchQueryRequest) -> Union[ServiceAccountsResponse, dict]:
        """ list service accounts

        Args:
            params (ServiceAccountSearchQueryRequest): {
                'query': 'dict (spaceone.api.core.v1.Query)',
                'service_account_id': 'str',
                'name': 'str',
                'provider': 'str',
                'project_id': 'str',
                'workspace_id': 'str',
                'domain_id': 'str',                     # required
                'user_projects': 'list'                 # from meta
            }

        Returns:
            ServiceAccountsResponse:
        """

        query = params.query or {}
        service_account_vos, total_count = self.service_account_mgr.list_service_accounts(query)

        service_accounts_info = [service_account_vo.to_dict() for service_account_vo in service_account_vos]
        return ServiceAccountsResponse(results=service_accounts_info, total_count=total_count)

    @transaction(scope="workspace_member:read")
    @append_query_filter(['workspace_id', 'domain_id', 'user_projects'])
    @append_keyword_filter(['service_account_id', 'name'])
    @set_query_page_limit(1000)
    @convert_model
    def stat(self, params: ServiceAccountStatQueryRequest) -> dict:
        """ stat service accounts

        Args:
            params (ServiceAccountStatQueryRequest): {
                'query': 'dict (spaceone.api.core.v1.StatisticsQuery)', # required
                'workspace_id': 'str',
                'domain_id': 'str',         # required
                'user_projects': 'list'     # from meta
            }

        Returns:
            dict: {
                'results': 'list',
                'total_count': 'int'
            }

        """

        query = params.query or {}
        return self.service_account_mgr.stat_service_accounts(query)



DEFAULT_PROVIDERS = [{
    "provider": "aws",
    "name": "AWS",
    "order": 1,
    "template": {
        "service_account": {
            "schema": {
                "type": "object",
                "properties": {
                    "account_id": {
                        "title": "Account ID",
                        "type": "string",
                        "minLength": 4
                    }
                },
                "required": ["account_id"]
            }
        }
    },
    "metadata": {
        "view": {
            "layouts": {
                "help:service_account:create": {
                    "name": "Creation Help",
                    "type": "markdown",
                    "options": {
                        "markdown": {
                            "en": (
                                "# Help for AWS Users\n"
                                "## Find Your AWS Account ID\n"
                                "Get your AWS Account ID.\n"
                                "[AWS Account ID](https://docs.aws.amazon.com/IAM/latest/UserGuide/console_account-alias.html)\n"
                                "## Get Your Assume role\n"
                                "Granting permissions to create temporary security credentials.\n"
                                "[AWS Assume Role](https://docs.aws.amazon.com/IAM/latest/UserGuide/id_roles_use_permissions-to-switch.html)\n"
                                "## Issue AWS Access Key \n"
                                "Get your AWS Access Key & AWS Secret Key\n"
                                "[AWS Access Key & AWS Secret Key](https://docs.aws.amazon.com/IAM/latest/UserGuide/id_credentials_access-keys.html#Using_CreateAccessKey)\n"
                            ),
                            "ko": (
                                "# AWS 이용자 가이드\n"
                                "## AWS 어카운트 아이디(Account ID) 찾기\n"
                                "사용자의 AWS 어카운트 아이디 AWS 콘솔(Console)에서 확인하기\n"
                                "[AWS Account ID](https://docs.aws.amazon.com/ko_kr/IAM/latest/UserGuide/console_account-alias.html)\n"
                                "## Assume role 획득하기\n"
                                "임시 보안 자격증명을 만들 수있는 권한을 부여하기.\n"
                                "[AWS Assume Role](https://docs.aws.amazon.com/ko_kr/IAM/latest/UserGuide/id_roles_use_permissions-to-switch.html)\n"
                                "## AWS Access Key 발급하기\n"
                                "AWS Access Key & AWS Secret Key 발급하기\n"
                                "[AWS Access Key & AWS Secret Key](https://docs.aws.amazon.com/ko_kr/IAM/latest/UserGuide/id_credentials_access-keys.html#Using_CreateAccessKey)\n"
                            ),

                        }
                    }
                }
            }
        }
    },
    "capability": {
        "general_service_account_schema": [
            "aws_assume_role"
        ],
        "trusted_service_account_schema": [
            "aws_access_key"
        ],
        "support_trusted_service_account": True,
        "supported_schema": [
            "aws_access_key"
        ]
    },
    "tags": {
        'color': '#FF9900',
        'icon': 'https://spaceone-custom-assets.s3.ap-northeast-2.amazonaws.com/console-assets/icons/aws.svg',
        'external_link_template': 'https://<%- data.account_id %>.signin.aws.amazon.com/console'
    }
}, {
    "provider": "google_cloud",
    "version": "v1",
    "name": "Google Cloud",
    "order": 2,
    "template": {
        "service_account": {
            "schema": {
                "type": "object",
                "properties": {
                    "project_id": {
                        "title": "Project ID",
                        "type": "string",
                        "minLength": 4
                    }
                },
                "required": ["project_id"]
            }
        }
    },
    "metadata": {
        "view": {
            "layouts": {
                "help:service_account:create": {
                    "name": "Creation Help",
                    "type": "markdown",
                    "options": {
                        "markdown": {
                            "en": (
                                "# Getting started with Google Cloud\n"
                                "## Identifying Your Project\n"
                                "Get your Project infos (Project Name, Project ID and Project number)\n"
                                "[Project Info](https://cloud.google.com/resource-manager/docs/creating-managing-projects#identifying_projects)\n"
                                "## Get Your Service Account Key(JSON)\n"
                                "Generate Your JSON Service Account Key.\n"
                                "[Service Account Key](https://cloud.google.com/docs/authentication/getting-started)\n"
                            ),
                            "ko": (
                                "# Google Cloud 시작 가이드\n"
                                "## Project 정보 확인하기\n"
                                "프로젝트 명, 프로젝트 아이디, 프로젝트 번호 등의 프로젝트 정보 확인하기\n"
                                "[Project Info](https://cloud.google.com/resource-manager/docs/creating-managing-projects?hl=ko#identifying_projects)\n"
                                "## 서비스 어카운트 키(JSON) 받기\n"
                                "JSON 포멧의 서비스 어카운트 키를 생성하기.\n"
                                "[Service Account Key](https://cloud.google.com/docs/authentication/getting-started?hl=ko)\n"
                            ),

                        }
                    }
                }
            }
        }
    },
    "capability": {
        "general_service_account_schema": [
            "google_cloud_project_id"
        ],
        "trusted_service_account_schema": [
            "google_oauth2_credentials"
        ],
        "support_trusted_service_account": True,
        "supported_schema": ["google_oauth2_credentials"]
    },
    "tags": {
        'color': '#4285F4',
        'icon': 'https://spaceone-custom-assets.s3.ap-northeast-2.amazonaws.com/console-assets/icons/google_cloud.svg',
        'external_link_template': 'https://console.cloud.google.com/home/dashboard?project=<%- data.project_id %>',
        'label': 'Google'
    }
}, {
    "provider": "azure",
    "name": "Azure",
    "order": 3,
    "template": {
        "service_account": {
            "schema": {
                "type": "object",
                "properties": {
                    "tenant_id": {
                        "title": "Tenant ID",
                        "type": "string",
                        "minLength": 4
                    },
                    "subscription_id": {
                        "title": "Subscription ID",
                        "type": "string",
                        "minLength": 4
                    }
                },
                "required": ["tenant_id", "subscription_id"]
            }
        }
    },
    "metadata": {
        "view": {
            "layouts": {
                "help:service_account:create": {
                    "name": "Creation Help",
                    "type": "markdown",
                    "options": {
                        "markdown": {
                            "en": (
                                "# Help for Azure Users\n"
                                "## Find Your Azure Subscription ID\n"
                                "Azure Subscription ID via CLI.\n"
                                "[Azure Subscription CLI](https://docs.microsoft.com/en-us/cli/azure/ext/account/account/subscription?view=azure-cli-latest)\n"
                                "Azure Subscription ID via PowerShell.\n"
                                "[Azure Subscription PowerShell](https://docs.microsoft.com/en-us/azure/active-directory/develop/howto-authenticate-service-principal-powershell)\n"
                                "Create Azure Subscription via Portal.\n"
                                "[Azure Subscription Portal](https://docs.microsoft.com/en-us/azure/active-directory/develop/howto-create-service-principal-portal)\n"
                                "## Find Your Azure Tenant ID\n"
                                "Azure Tenant ID via CLI.\n"
                                "[Azure Tenant CLI](https://docs.microsoft.com/en-us/cli/azure/ext/account/account/tenant?view=azure-cli-latest)\n"
                                "Azure Tenant ID via PowerShell.\n"
                                "[Azure Tenant PowerShell](https://docs.microsoft.com/en-us/powershell/module/az.accounts/get-aztenant?view=azps-5.0.0)\n"
                                "## Get Your Client Secret and ID\n"
                                "Check Client Secret via Portal.\n"
                                "[Azure Client Secret Portal](https://docs.microsoft.com/en-us/azure/active-directory/develop/howto-create-service-principal-portal)\n"
                            ),
                            "ko": (
                                "# Azure 이용자 가이드\n"
                                "## Azure 구독 아이디(Subscription ID) 찾기\n"
                                "CLI에서 사용자의 구독 아이디 확인하기.\n"
                                "[Azure Subscription CLI](https://docs.microsoft.com/en-us/cli/azure/ext/account/account/subscription?view=azure-cli-latest)\n"
                                "PowerShell에서 사용자의 구독 아이디 확인하기.\n"
                                "[Azure Subscription PowerShell](https://docs.microsoft.com/en-us/azure/active-directory/develop/howto-authenticate-service-principal-powershell)\n"
                                "포털에서 사용자의 구독 아이디 확인하기.\n"
                                "[Azure Subscription Portal](https://docs.microsoft.com/en-us/azure/active-directory/develop/howto-create-service-principal-portal)\n"
                                "## Azure 테넌트 아이디(Tenant ID) 찾기\n"
                                "CLI에서 사용자의 테넌트 아이디 확인하기.\n"
                                "[Azure Tenant CLI](https://docs.microsoft.com/en-us/cli/azure/ext/account/account/tenant?view=azure-cli-latest)\n"
                                "PowerShell에서 사용자의 테넌트 아이디 확인하기.\n"
                                "[Azure Tenant PowerShell](https://docs.microsoft.com/en-us/powershell/module/az.accounts/get-aztenant?view=azps-5.0.0)\n"
                                "## 사용자의 클라이언트 시크릿 정보(Client Secret&ID) 가져오기\n"
                                "포털에서 사용자의 클라이언트 시크릿 정보 확인하기.\n"
                                "[Azure Client Secret Portal](https://docs.microsoft.com/en-us/azure/active-directory/develop/howto-create-service-principal-portal)\n"
                            ),

                        }
                    }
                }
            }
        }
    },
    "capability": {
        "general_service_account_schema": [
            "azure_subscription_id"
        ],
        "trusted_service_account_schema": [
            "azure_credentials",
        ],
        "support_trusted_service_account": True,
        "supported_schema": ["azure_client_secret"]
    },
    "tags": {
        'color': '#00BCF2',
        'icon': 'https://spaceone-custom-assets.s3.ap-northeast-2.amazonaws.com/console-assets/icons/azure.svg'
    }
}]


# -*- coding: utf-8 -*-

import bcrypt


class PasswordCipher:
    @staticmethod
    def __encoder(password: str) -> str:
        return str(password).encode("utf-8")

    def hashpw(self, password: str) -> bytes:
        return bcrypt.hashpw(self.__encoder(password), bcrypt.gensalt())

    def checkpw(self, password, hashed) -> bool:
        return bcrypt.checkpw(self.__encoder(password), hashed)



from typing import Union, List
from pydantic import BaseModel

__all__ = [
    "UserGroupCreateRequest",
    "UserGroupUpdateRequest",
    "UserGroupDeleteRequest",
    "UserGroupAddUsersRequest",
    "UserGroupRemoveUsersRequest",
    "UserGroupGetRequest",
    "UserGroupSearchQueryRequest",
    "UserGroupStatQueryRequest",
]


class UserGroupCreateRequest(BaseModel):
    name: str
    tags: Union[dict, None] = None
    workspace_id: str
    domain_id: str


class UserGroupUpdateRequest(BaseModel):
    user_group_id: str
    name: Union[str, None] = None
    tags: Union[dict, None] = None
    workspace_id: str
    domain_id: str


class UserGroupDeleteRequest(BaseModel):
    user_group_id: str
    workspace_id: str
    domain_id: str


class UserGroupAddUsersRequest(BaseModel):
    user_group_id: str
    users: List[str]
    workspace_id: str
    domain_id: str


class UserGroupRemoveUsersRequest(BaseModel):
    user_group_id: str
    users: List[str]
    workspace_id: str
    domain_id: str


class UserGroupGetRequest(BaseModel):
    user_group_id: str
    workspace_id: str
    domain_id: str


class UserGroupSearchQueryRequest(BaseModel):
    query: Union[dict, None] = None
    user_group_id: Union[str, None] = None
    name: Union[str, None] = None
    user_id: Union[str, None] = None
    workspace_id: Union[str, None] = None
    domain_id: str


class UserGroupStatQueryRequest(BaseModel):
    query: dict
    workspace_id: Union[str, None] = None
    domain_id: str



from mongoengine import *
from spaceone.core.model.mongo_model import MongoModel


class RoleBinding(MongoModel):
    role_binding_id = StringField(max_length=40, generate_id='rb', unique=True)
    role_type = StringField(
        max_length=20,
        choices=('SYSTEM_ADMIN', 'DOMAIN_ADMIN', 'WORKSPACE_OWNER', 'WORKSPACE_MEMBER')
    )
    user_id = StringField(max_length=255)
    role_id = StringField(max_length=40)
    permission_group = StringField(max_length=40, choices=('DOMAIN', 'WORKSPACE'), default='WORKSPACE')
    workspace_id = StringField(max_length=40)
    domain_id = StringField(max_length=40)
    created_at = DateTimeField(auto_now_add=True)

    meta = {
        'updatable_fields': [
            'role_id',
            'role_type',
        ],
        'minimal_fields': [
            'role_binding_id',
            'role_type',
            'user_id',
            'role_id',
            'workspace_id'
        ],
        'indexes': [
            'role_type',
            'user_id',
            'role_id',
            'permission_group',
            'workspace_id',
            'domain_id',
        ]
    }


import logging
from typing import Union

from spaceone.core.service import *
from spaceone.core.service.utils import *

from spaceone.identity.error.error_user_group import *
from spaceone.identity.manager.user_manager import UserManager
from spaceone.identity.manager.user_group_manager import UserGroupManager
from spaceone.identity.model.user_group.request import *
from spaceone.identity.model.user_group.response import *

_LOGGER = logging.getLogger(__name__)


@authentication_handler
class UserGroupService(BaseService):
    service = "identity"
    resource = "UserGroup"
    permission_group = "Workspace"

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.user_group_mgr = UserGroupManager()

    @transaction(scope="workspace_owner:write")
    @convert_model
    def create(self, params: UserGroupCreateRequest) -> Union[UserGroupResponse, dict]:
        """Create user group
        Args:
            params (dict): {
                'name': 'str',          # required
                'tags': 'dict',         # required
                'workspace_id': 'str',  # required
                'domain_id': 'str'      # required
            }
        """
        user_group_vo = self.user_group_mgr.create_user_group(params.dict())
        return UserGroupResponse(**user_group_vo.to_dict())

    @transaction(scope="workspace_owner:write")
    @convert_model
    def update(self, params: UserGroupUpdateRequest) -> Union[UserGroupResponse, dict]:
        """Update user group
        Args:
            params (dict): {
                'user_group_id': 'str', # required
                'name': 'str',
                'tags': 'dict',
                'workspace_id': 'str',  # required
                'domain_id': 'str'      # required
            }
        """
        user_group_vo = self.user_group_mgr.get_user_group(
            params.user_group_id,
            params.workspace_id,
            params.domain_id,
        )

        user_group_vo = self.user_group_mgr.update_user_group_by_vo(
            params.dict(exclude_unset=True), user_group_vo
        )

        return UserGroupResponse(**user_group_vo.to_dict())

    @transaction(scope="workspace_owner:write")
    @convert_model
    def delete(self, params: UserGroupDeleteRequest) -> None:
        """Delete user group
        Args:
            params (dict): {
                'user_group_id': 'str', # required
                'workspace_id': 'str',  # required
                'domain_id': 'str'      # required
            }
        Returns:
            None
        """

        user_group_vo = self.user_group_mgr.get_user_group(
            params.user_group_id, params.workspace_id, params.domain_id
        )
        self.user_group_mgr.delete_user_group_by_vo(user_group_vo)

    @transaction(scope="workspace_owner:write")
    @convert_model
    def add_users(
        self, params: UserGroupAddUsersRequest
    ) -> Union[UserGroupResponse, dict]:
        """Add users to user group
        Args:
            params (dict): {
                'user_group_id': 'str',     # required
                'users': 'list(str)',       # required
                'workspace_id': 'str',      # required
                'domain_id': 'str'          # required
            }
        Returns:
            UserGroupResponse:
        """
        user_group_vo = self.user_group_mgr.get_user_group(
            params.user_group_id, params.workspace_id, params.domain_id
        )

        user_mgr = UserManager()
        users_vo = user_mgr.filter_users(
            user_id=params.users, domain_id=params.domain_id
        )

        if users_vo.count() != len(params.users):
            raise ERROR_USERS_NOT_FOUND(
                users=list(set(params.users) - set(users_vo.values_list("user_id")))
            )

        params.users = list(set(user_group_vo.users + params.users))

        user_group_vo = self.user_group_mgr.update_user_group_by_vo(
            params.dict(exclude_unset=True), user_group_vo
        )

        return UserGroupResponse(**user_group_vo.to_dict())

    @transaction(scope="workspace_owner:write")
    @convert_model
    def remove_users(
        self, params: UserGroupRemoveUsersRequest
    ) -> Union[UserGroupResponse, dict]:
        """Remove users from user group
        Args:
            params (dict): {
                'user_group_id': 'str',     # required
                'users': 'list(str)',       # required
                'workspace_id': 'str',      # required
                'domain_id': 'str'          # required
            }
        Returns:
            UserGroupResponse:
        """
        user_group_vo = self.user_group_mgr.get_user_group(
            params.user_group_id, params.workspace_id, params.domain_id
        )

        user_mgr = UserManager()
        params.users = list(set(user_group_vo.users) - set(params.users))
        users_vo = self.user_group_mgr.update_user_group_by_vo(
            params.dict(exclude_unset=True), user_group_vo
        )
        return UserGroupResponse(**users_vo.to_dict())

    @transaction(scope="workspace_owner:read")
    @convert_model
    def get(self, params: UserGroupGetRequest) -> Union[UserGroupResponse, dict]:
        """Get user group
        Args:
            params (dict): {
                'user_group_id': 'str', # required
                'workspace_id': 'str',  # required
                'domain_id': 'str'      # required
            }
        Returns:
            UserGroupResponse:
        """

        user_group_vo = self.user_group_mgr.get_user_group(
            params.user_group_id, params.workspace_id, params.domain_id
        )
        return UserGroupResponse(**user_group_vo.to_dict())

    @transaction(scope="workspace_owner:read")
    @append_query_filter(
        [
            "user_group_id",
            "name",
            "user_id",
            "workspace_id",
            "domain_id",
        ]
    )
    @append_keyword_filter(["trusted_account_id", "name"])
    @convert_model
    def list(
        self, params: UserGroupSearchQueryRequest
    ) -> Union[UserGroupsResponse, dict]:
        """List user groups
        Args:
            params (dict): {
                'query': 'dict (spaceone.api.core.v1.Query)',
                'user_group_id': 'str',
                'name': 'str',
                'user_id': 'str',
                'workspace_id': 'str',
                'domain_id': 'str'      # required
            }
        Returns:
            UserGroupsResponse:
        """
        query = params.query or {}
        user_group_vos, total_count = self.user_group_mgr.list_user_groups(query)

        user_groups_info = [user_group_vo.to_dict() for user_group_vo in user_group_vos]
        return UserGroupsResponse(results=user_groups_info, total_count=total_count)

    @transaction(scope="workspace_owner:read")
    @append_query_filter(["workspace_id", "domain_id"])
    @append_keyword_filter(["user_group_id", "name"])
    @convert_model
    def stat(self, params: UserGroupStatQueryRequest) -> dict:
        """Stat user groups
        Args:
            params (dict): {
                'query': 'dict (spaceone.api.core.v1.StatisticsQuery)',
                'workspace_id': 'str',
                'domain_id': 'str'      # required
            }
        Returns:
            dict: {
                'results': 'list',
                'total_count': 'int'
            }
        """
        query = params.query or {}
        return self.user_group_mgr.stat_user_group(query)



import logging
from typing import Union

from spaceone.core.service import *
from spaceone.core.service.utils import *

from spaceone.identity.model.trusted_account.request import *
from spaceone.identity.model.trusted_account.response import *
from spaceone.identity.manager.schema_manager import SchemaManager
from spaceone.identity.manager.trusted_account_manager import TrustedAccountManager
from spaceone.identity.manager.workspace_manager import WorkspaceManager

_LOGGER = logging.getLogger(__name__)


class TrustedAccountService(BaseService):

    service = "identity"
    resource = "TrustedAccount"
    permission_group = "COMPOUND"

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.trusted_account_mgr = TrustedAccountManager()

    @transaction(scope="workspace_owner:write")
    @convert_model
    def create(self, params: TrustedAccountCreateRequest) -> Union[TrustedAccountResponse, dict]:
        """ create trusted account

         Args:
            params (TrustedAccountCreateRequest): {
                'name': 'str',          # required
                'data': 'dict',         # required
                'provider': 'str',      # required
                'tags': 'dict',
                'permission_group': 'str',         # required
                'workspace_id': 'str',
                'domain_id': 'str'      # required
            }

        Returns:
            TrustedAccountResponse:
        """

        # Check workspace
        if params.permission_group == 'WORKSPACE':
            workspace_mgr = WorkspaceManager()
            workspace_mgr.get_workspace(params.workspace_id, params.domain_id)
        else:
            params.workspace_id = '*'

        # Check data by schema
        schema_mgr = SchemaManager()
        schema_mgr.validate_data_by_schema(
            params.provider, params.domain_id, 'TRUSTED_ACCOUNT', params.data
        )

        trusted_account_vo = self.trusted_account_mgr.create_trusted_account(params.dict())
        return TrustedAccountResponse(**trusted_account_vo.to_dict())

    @transaction(scope="workspace_owner:write")
    @convert_model
    def update(self, params: TrustedAccountUpdateRequest) -> Union[TrustedAccountResponse, dict]:
        """ update trusted account

         Args:
            params (TrustedAccountUpdateRequest): {
                'trusted_account_id': 'str',    # required
                'name': 'str',
                'data': 'dict',
                'tags': 'dict',
                'workspace_id': 'str',
                'domain_id': 'str'                      # required
            }

        Returns:
            TrustedAccountResponse:
        """

        trusted_account_vo = self.trusted_account_mgr.get_trusted_account(
            params.trusted_account_id, params.domain_id, params.workspace_id
        )

        if params.data:
            # Check data by schema
            schema_mgr = SchemaManager()
            schema_mgr.validate_data_by_schema(
                trusted_account_vo.provider, params.domain_id, 'TRUSTED_ACCOUNT', params.data
            )

        trusted_account_vo = self.trusted_account_mgr.update_trusted_account_by_vo(
            params.dict(exclude_unset=True), trusted_account_vo
        )

        return TrustedAccountResponse(**trusted_account_vo.to_dict())

    @transaction(scope="workspace_owner:write")
    @convert_model
    def delete(self, params: TrustedAccountDeleteRequest) -> None:
        """ delete trusted account

         Args:
            params (TrustedAccountDeleteRequest): {
                'trusted_account_id': 'str',    # required
                'workspace_id': 'str',
                'domain_id': 'str'                      # required
            }

        Returns:
            None
        """

        trusted_account_vo = self.trusted_account_mgr.get_trusted_account(
            params.trusted_account_id, params.domain_id, params.workspace_id
        )

        self.trusted_account_mgr.delete_trusted_account_by_vo(trusted_account_vo)

    @transaction(scope="workspace_member:read")
    @convert_model
    def get(self, params: TrustedAccountGetRequest) -> Union[TrustedAccountResponse, dict]:
        """ get trusted account

         Args:
            params (TrustedAccountGetRequest): {
                'trusted_account_id': 'str',    # required
                'workspace_id': 'str',
                'domain_id': 'str'                      # required
            }

        Returns:
            TrustedAccountResponse:
        """

        trusted_account_vo = self.trusted_account_mgr.get_trusted_account(
            params.trusted_account_id, params.domain_id, params.workspace_id
        )

        return TrustedAccountResponse(**trusted_account_vo.to_dict())

    @transaction(scope="workspace_member:read")
    @append_query_filter([
        'trusted_account_id', 'name', 'provider', 'permission_group', 'workspace_id', 'domain_id'
    ])
    @append_keyword_filter(['trusted_account_id', 'name'])
    @convert_model
    def list(self, params: TrustedAccountSearchQueryRequest) -> Union[TrustedAccountsResponse, dict]:
        """ list trusted accounts

        Args:
            params (TrustedAccountSearchQueryRequest): {
                'query': 'dict (spaceone.api.core.v1.Query)',
                'trusted_account_id': 'str',
                'name': 'str',
                'provider': 'str',
                'permission_group': 'str',
                'workspace_id': 'str',
                'domain_id': 'str',         # required
            }

        Returns:
            TrustedAccountsResponse:
        """

        query = params.query or {}
        trusted_account_vos, total_count = self.trusted_account_mgr.list_trusted_accounts(query)

        trusted_accounts_info = [trusted_account_vo.to_dict() for trusted_account_vo in trusted_account_vos]
        return TrustedAccountsResponse(results=trusted_accounts_info, total_count=total_count)

    @transaction(scope="workspace_member:read")
    @append_query_filter(['workspace_id', 'domain_id'])
    @append_keyword_filter(['trusted_account_id', 'name'])
    @convert_model
    def stat(self, params: TrustedAccountStatQueryRequest) -> dict:
        """ stat trusted accounts

        Args:
            params (TrustedAccountStatQueryRequest): {
                'query': 'dict (spaceone.api.core.v1.StatisticsQuery)', # required
                'workspace_id': 'str',
                'domain_id': 'str',         # required
                'user_workspaces': 'list'   # from meta
            }

        Returns:
            dict: {
                'results': 'list',
                'total_count': 'int'
            }

        """

        query = params.query or {}
        return self.trusted_account_mgr.stat_trusted_accounts(query)



import logging
import time
from typing import Tuple

from spaceone.core.auth.jwt.jwt_util import JWTUtil
from spaceone.core import utils

from spaceone.identity.error import ERROR_GENERATE_KEY_FAILURE

_LOGGER = logging.getLogger(__name__)


class KeyGenerator:
    def __init__(
        self,
        prv_jwk: dict,
        domain_id: str,
        audience: str,
        api_key_id: str = None,
        refresh_prv_jwk: dict = None,
    ):
        self.prv_jwk = prv_jwk
        self.domain_id = domain_id
        self.audience = audience
        self.token_id = api_key_id or utils.random_string(16)
        self.refresh_prv_jwk = refresh_prv_jwk

        self._check_parameter()

    def _check_parameter(self):
        if self.prv_jwk is None:
            raise ERROR_GENERATE_KEY_FAILURE()
        if self.domain_id is None:
            raise ERROR_GENERATE_KEY_FAILURE()

    def generate_api_key(self, api_key_id: str, expired: int) -> str:
        payload = {
            "iss": "spaceone.identity",
            "typ": "API_KEY",  # API_KEY | ACCESS_TOKEN | REFRESH_TOKEN
            "own": "USER",  # USER | APP
            "did": self.domain_id,
            "aud": self.audience,
            "exp": expired,
            "jti": self.token_id,
            "iat": int(time.time()),
            "ver": "2.0",
        }

        self._print_key(payload)
        return JWTUtil.encode(payload, self.prv_jwk)

    def generate_access_token(self, expired: int, api_permissions: list = None) -> str:
        payload = {
            "iss": "spaceone.identity",
            "typ": "ACCESS_TOKEN",
            "own": "USER",  # USER | APP
            "did": self.domain_id,
            "aud": self.audience,
            "exp": int(time.time() + expired),
            "jti": self.token_id,
            "iat": int(time.time()),
            "ver": "2.0",
        }

        if api_permissions:
            payload["api_permissions"] = api_permissions

        return JWTUtil.encode(payload, self.prv_jwk)

    def generate_refresh_token(self, expired: int, ttl: int) -> str:
        payload = {
            "iss": "spaceone.identity",
            "typ": "REFRESH_TOKEN",
            "own": "USER",  # USER | APP
            "did": self.domain_id,
            "aud": self.audience,
            "exp": int(time.time() + expired),
            "jti": self.token_id,
            "iat": int(time.time()),
            "ttl": ttl,
            "ver": "2.0",
        }

        return JWTUtil.encode(payload, self.refresh_prv_jwk)

    @staticmethod
    def _print_key(payload: dict):
        _LOGGER.debug(
            f"[KeyGenerator._print_key] generated payload. ( "
            f'iss: {payload.get("iss")}, '
            f'typ: {payload.get("typ")}, '
            f'own: {payload.get("own")}, '
            f'did: {payload.get("did")}, '
            f'aud: {payload.get("aud")}, '
            f'exp: {payload.get("exp")}, '
            f'iat: {payload.get("iat")}, '
            f'ttl: {payload.get("ttl")}, '
            f'jti: {payload.get("jti")}, '
            f'ver: {payload.get("ver")} )'
        )



from mongoengine import *
from spaceone.core.model.mongo_model import MongoModel


class MFA(EmbeddedDocument):
    state = StringField(
        max_length=20, choices=("ENABLED", "DISABLED"), default="DISABLED"
    )
    mfa_type = StringField(max_length=20)
    options = DictField()

    def to_dict(self):
        return dict(self.to_mongo())


class User(MongoModel):
    user_id = StringField(max_length=40, unique_with="domain_id", required=True)
    password = BinaryField(default=None)
    name = StringField(max_length=128, default="")
    state = StringField(max_length=20, default="PENDING", choices=("ENABLED", "DISABLED", "PENDING"))
    email = StringField(max_length=255, default="")
    email_verified = BooleanField(default=False)
    auth_type = StringField(max_length=20, choices=("LOCAL", "EXTERNAL"))
    role_type = StringField(
        max_length=20,
        default="USER",
        choices=('SYSTEM', 'SYSTEM_ADMIN', 'DOMAIN_ADMIN', 'WORKSPACE_OWNER', 'WORKSPACE_MEMBER', 'USER'),
    )
    mfa = EmbeddedDocumentField(MFA)
    required_actions = ListField(StringField(choices=("UPDATE_PASSWORD",)), default=[])
    language = StringField(max_length=7, default="en")
    timezone = StringField(max_length=50, default="UTC")
    api_key_count = IntField(default=0)
    tags = DictField(Default=None)
    domain_id = StringField(max_length=40)
    created_at = DateTimeField(auto_now_add=True)
    last_accessed_at = DateTimeField(default=None, null=True)

    meta = {
        "updatable_fields": [
            "password",
            "name",
            "state",
            "email",
            "email_verified",
            'role_type',
            "mfa",
            "language",
            "timezone",
            "required_actions",
            "tags",
            "api_key_count",
            "last_accessed_at",
        ],
        "minimal_fields": [
            "user_id",
            "name",
            "state",
            'auth_type',
            "role_type"
        ],
        "ordering": [
            "name",
            "user_id"
        ],
        "indexes": [
            "state",
            "auth_type",
            "role_type",
        ],
    }



from spaceone.core.pygrpc import BaseAPI
from spaceone.api.identity.v2 import api_key_pb2, api_key_pb2_grpc
from spaceone.identity.service.api_key_service import APIKeyService


class APIKey(BaseAPI, api_key_pb2_grpc.APIKeyServicer):
    pb2 = api_key_pb2
    pb2_grpc = api_key_pb2_grpc

    def create(self, request, context):
        params, metadata = self.parse_request(request, context)
        api_key_svc = APIKeyService(metadata)
        response: dict = api_key_svc.create(params)
        return self.dict_to_message(response)

    def update(self, request, context):
        params, metadata = self.parse_request(request, context)
        api_key_svc = APIKeyService(metadata)
        response: dict = api_key_svc.update(params)
        return self.dict_to_message(response)

    def enable(self, request, context):
        params, metadata = self.parse_request(request, context)
        api_key_svc = APIKeyService(metadata)
        response: dict = api_key_svc.enable(params)
        return self.dict_to_message(response)

    def disable(self, request, context):
        params, metadata = self.parse_request(request, context)
        api_key_svc = APIKeyService(metadata)
        response: dict = api_key_svc.disable(params)
        return self.dict_to_message(response)

    def delete(self, request, context):
        params, metadata = self.parse_request(request, context)
        api_key_svc = APIKeyService(metadata)
        api_key_svc.delete(params)
        return self.empty()

    def get(self, request, context):
        params, metadata = self.parse_request(request, context)
        api_key_svc = APIKeyService(metadata)
        response: dict = api_key_svc.get(params)
        return self.dict_to_message(response)

    def list(self, request, context):
        params, metadata = self.parse_request(request, context)
        api_key_svc = APIKeyService(metadata)
        response: dict = api_key_svc.list(params)
        return self.dict_to_message(response)

    def stat(self, request, context):
        params, metadata = self.parse_request(request, context)
        api_key_svc = APIKeyService(metadata)
        response: dict = api_key_svc.stat(params)
        return self.dict_to_message(response)
